### Data processing Illumina paired end with DADA2, update ver 1.8.0, 19/07/2018 ###
# Big data workflow. Each step is self consistent and it is possible to stop after each step
# R1 and R2 fastq reads must be in two different directories
# Create an R project in the working directory with the following subdirectories:
#                                         - R1 (contains R1)
#                                         - R2 (contains R2)
#                                         - dada.scripts (contains R.script for dada2)
#                                         - taxonomy (contains the databases to assign taxonomy)

message("STEP1: QUALITY CHECK")
message("Upload libraries")
library(Rcpp)
library(dada2); packageVersion("dada2")

message("File parsing")
pathF <- "R1"
pathR <- "R2"
fastqFsq <- sort(list.files(pathF, pattern=".R1.fastq", full.names=TRUE))
fastqRsq <- sort(list.files(pathR, pattern=".R2.fastq", full.names=TRUE))
sample.names <- sapply(strsplit(basename(fastqFsq), ".R1."), `[`, 1) # CHANGE WITH SEQUENCE NAMES. Extract sample names. This can be variable in different projects



# Quality profile
#dir.create(file.path(pathF, "quality"))
#tiff(filename = file.path(pathF, "quality/QualityProfile_ForwardReads.tif"), width=2160, height=1440, units="px", pointsize=12)
#plotQualityProfile(fastqFsq)
#dev.off()

#dir.create(file.path(pathR, "quality"))
#tiff(filename = file.path(pathR, "quality/QualityProfile_ReverseReads.tif"), width=2160, height=1440, units="px", pointsize=12)
#plotQualityProfile(fastqRsq)
#dev.off()


message("STEP2: FILTERING")
### filtering is performed with 6 different parameters and the best results is selected
message("Upload libraries")
library(dada2); packageVersion("dada2")

message("File parsing")
pathF <- "R1"
pathR <- "R2"
fastqFs <- sort(list.files(pathF, pattern=".R1.fastq"))
fastqRs <- sort(list.files(pathR, pattern=".R2.fastq"))
dir.create("track_reads")


dir.create(file.path(pathF, "filtered1"))
filtpathF1 <- file.path(pathF, "filtered1")
#dir.create(file.path(pathF, "filtered2"))
#filtpathF2 <- file.path(pathF, "filtered2")
#dir.create(file.path(pathF, "filtered3"))
#filtpathF3 <- file.path(pathF, "filtered3")
#dir.create(file.path(pathF, "filtered4"))
#filtpathF4 <- file.path(pathF, "filtered4")
#dir.create(file.path(pathF, "filtered5"))
#filtpathF5 <- file.path(pathF, "filtered5")
#dir.create(file.path(pathF, "filtered6"))
#filtpathF6 <- file.path(pathF, "filtered6")

dir.create(file.path(pathR, "filtered1"))
filtpathR1 <- file.path(pathR, "filtered1")
#dir.create(file.path(pathR, "filtered2"))
#filtpathR2 <- file.path(pathR, "filtered2")
#dir.create(file.path(pathR, "filtered3"))
#filtpathR3 <- file.path(pathR, "filtered3")
#dir.create(file.path(pathR, "filtered4"))
#filtpathR4 <- file.path(pathR, "filtered4")
#dir.create(file.path(pathR, "filtered5"))
#filtpathR5 <- file.path(pathR, "filtered5")
#dir.create(file.path(pathR, "filtered6"))
#filtpathR6 <- file.path(pathR, "filtered6")

if(length(fastqFs) != length(fastqRs)) stop("Forward and reverse files do not match.")

message("Filtering")
#THESE PARAMETERS ARENT OPTIMAL FOR ALL DATASETS
out1 <- filterAndTrim(fwd=file.path(pathF, fastqFs), filt=file.path(filtpathF1, fastqFs),
              rev=file.path(pathR, fastqRs), filt.rev=file.path(filtpathR1, fastqRs),
              truncLen=c(180,150),
              maxN=0, maxEE=c(0.5,0.5), trimLeft=c(10,20), rm.phix=TRUE,
              compress=FALSE, multithread=TRUE) # Adjust filtering parameters if needed. On Windows set multithread=FALSE
write.table(out1, "track_reads/filter1.txt")

#out2 <- filterAndTrim(fwd=file.path(pathF, fastqFs), filt=file.path(filtpathF2, fastqFs),
#             rev=file.path(pathR, fastqRs), filt.rev=file.path(filtpathR2, fastqRs),
#              truncLen=c(240,200),
#              maxN=0, maxEE=c(1,1), trimLeft=c(10,10), rm.phix=TRUE,
#              compress=FALSE, multithread=TRUE) # Adjust filtering parameters if needed. On Windows set multithread=FALSE
#write.table(out2, "track_reads/filter2.txt")

#out3 <- filterAndTrim(fwd=file.path(pathF, fastqFs), filt=file.path(filtpathF3, fastqFs),
#              rev=file.path(pathR, fastqRs), filt.rev=file.path(filtpathR3, fastqRs),
#              truncLen=c(240,240),
#              maxN=0, maxEE=c(0.5,0.5), trimLeft=c(10,10), rm.phix=TRUE,
#              compress=FALSE, multithread=FALSE) # Adjust filtering parameters if needed. On Windows set multithread=FALSE
#write.table(out3, "track_reads/filter3.txt")

#out4 <- filterAndTrim(fwd=file.path(pathF, fastqFs), filt=file.path(filtpathF4, fastqFs),
#              rev=file.path(pathR, fastqRs), filt.rev=file.path(filtpathR4, fastqRs),
#              truncLen=c(240,240),
#              maxN=0, maxEE=c(1,1), trimLeft=c(10,10), rm.phix=TRUE,
#              compress=FALSE, multithread=FALSE) # Adjust filtering parameters if needed. On Windows set multithread=FALSE
#write.table(out4, "track_reads/filter4.txt")

#out5 <- filterAndTrim(fwd=file.path(pathF, fastqFs), filt=file.path(filtpathF5, fastqFs),
#              rev=file.path(pathR, fastqRs), filt.rev=file.path(filtpathR5, fastqRs),
#              truncLen=c(240,240),
#              maxN=0, maxEE=c(0.5,0.5), trimLeft=c(5,5), rm.phix=TRUE,
#              compress=FALSE, multithread=FALSE) # Adjust filtering parameters if needed. On Windows set multithread=FALSE
#write.table(out5, "track_reads/filter5.txt")

#out6 <- filterAndTrim(fwd=file.path(pathF, fastqFs), filt=file.path(filtpathF6, fastqFs),
#                      rev=file.path(pathR, fastqRs), filt.rev=file.path(filtpathR6, fastqRs),
#                      truncLen=c(240,200),
#                      maxN=0, maxEE=c(0.5,0.5), trimLeft=c(5,5), rm.phix=TRUE,
#                      compress=FALSE, multithread=FALSE) # Adjust filtering parameters if needed. On Windows set multithread=FALSE
#write.table(out6, "track_reads/filter6.txt")

message("STEP3: INFER SEQUENCE VARIANTS")
### Choose the proper filtered reads from STEP2
# WRITE FILTERING PARAMETERS HERE: filter1=truncLen=c(240,200),maxN=0,maxEE=c(0.5,0.5),trimLeft=c(10,10) (add filter to downsteram directories and files)
# If multiple runs have been performed this step has to be done for each run indipendently
# Move filtered reads of each run in a separated directory and work there
# This portion of the workflow should be performed on a run-by-run basis, as error rates can differ between runs
message("File parsing")
pathF <- "R1"
pathR <- "R2"
filtpathF <- file.path(pathF, "filtered1")
filtpathR <- file.path(pathR, "filtered1")
filtFs <- list.files(filtpathF, pattern=".R1.fastq", full.names = TRUE)
filtRs <- list.files(filtpathR, pattern=".R2.fastq", full.names = TRUE)
sample.names <- sapply(strsplit(basename(filtFs), ".R1.fastq"), `[`, 1)
sample.namesR <- sapply(strsplit(basename(filtRs), ".R2.fastq"), `[`, 1)
if(!identical(sample.names, sample.namesR)) stop("Forward and reverse files do not match.")
names(filtFs) <- sample.names
names(filtRs) <- sample.names
set.seed(100)

message("Learn the error rates and plot them")
dir.create(file.path(filtpathF, "error.rates_filter1"))
dir.create(file.path(filtpathR, "error.rates_filter1"))

errF <- learnErrors(filtFs, nbases=1e8, multithread=TRUE, randomize=TRUE)
#tiff(filename = file.path(filtpathF, "error.rates_filter1/ErrorRates_ForwardReads_filter1.tif"), width=1080, height=720, units="px", pointsize=12)
plotErrors(errF, nominalQ=TRUE)
#dev.off()

errR <- learnErrors(filtRs, nbases=1e8, multithread=TRUE, randomize=TRUE)
 #tiff(filename = file.path(filtpathR, "error.rates_filter1/ErrorRates_ReverseReads_filter1.tif"), width=1080, height=720, units="px", pointsize=12)
plotErrors(errR, nominalQ=TRUE)
#dev.off()

message("Sample inference and merger of paired-end reads")
mergers <- vector("list", length(sample.names))
names(mergers) <- sample.names
for(sam in sample.names) {
  cat("Processing:", sam, "\n")
  derepF <- derepFastq(filtFs[[sam]])
  ddF <- dada(derepF, err=errF, multithread=TRUE)
  derepR <- derepFastq(filtRs[[sam]])
  ddR <- dada(derepR, err=errR, multithread=TRUE)
  merger <- mergePairs(ddF, derepF, ddR, derepR)
  mergers[[sam]] <- merger
}
rm(derepF); rm(derepR)
# Construct sequence table
dir.create("out_filter1")
seqtab <- makeSequenceTable(mergers)
saveRDS(seqtab, "out_filter1/seqtab_filter1.rds")
getN <- function(x) sum(getUniques(x))
merged <- sapply(mergers, getN)
write.table(merged, "track_reads/merged_filter1.txt")

message("STEP4: MERGE RUNS, REMOVE CHIMERAS, ASSIGN TAXONOMY ###")

library(dada2); packageVersion("dada2")

# ## Multilpe runs
# # Merge runs
# st1 <- readRDS("path/to/run1/output/seqtab.rds")
# st2 <- readRDS("path/to/run2/output/seqtab.rds")
# st.all <- mergeSequenceTables(st1, st2, st3)

# # Remove chimeras
# seqtab <- removeBimeraDenovo(st.all, method="consensus", multithread=TRUE)
# saveRDS(seqtab, file.path("seqtab_final.rds"))

## One run
# Open seqtab
st.all <- readRDS("out_filter1/seqtab_filter1.rds")

message("Remove chimeras")
seqtab <- removeBimeraDenovo(st.all, method="consensus", multithread=TRUE)
saveRDS(seqtab, "out_filter1/seqtab_filter1_nochim.rds")
nochim <- rowSums(seqtab)
write.table(nochim, "track_reads/nochim_filter1.txt")

message("Create table to track reads that passed filtering, merging, chimera check")
library(data.table)
out <- read.table("track_reads/filter1.txt")
merged <- read.table("track_reads/merged_filter1.txt")
track <- cbind(out, merged, nochim)
colnames(track) <- c("input", "filtered", "merged", "nonchim")
sample.names <- sapply(strsplit(rownames(track), "_"), `[`, 1)
rownames(track) <- sample.names
track1 <- setDT(as.data.frame(track), keep.rownames = TRUE)
colnames(track1)[colnames(track1) == c("rn")] <- c("sample")
write.table(track1, "track_reads/track_sequences_filter1.txt", quote = FALSE, sep = "\t", row.names = FALSE)

message("Assign taxonomy with Silva")
taxa.silva <- assignTaxonomy(seqtab, file.path("taxonomy/silva_nr_v132_train_set.fa.gz"), minBoot = 80, multithread=TRUE, verbose=TRUE)
species.silva <- addSpecies(taxa.silva, file.path("taxonomy/silva_species_assignment_v132.fa.gz"), verbose=TRUE) # Assign species
saveRDS(taxa.silva, file.path("out_filter1/taxa.silva.rds"))
saveRDS(species.silva, file.path("out_filter1/species.silva.rds"))

message("Assign taxonomy with RDP")
taxa.rdp <- assignTaxonomy(seqtab, file.path("taxonomy/rdp_train_set_16.fa.gz"), minBoot = 80, multithread=TRUE, verbose=TRUE)
species.rdp <- addSpecies(taxa.rdp, file.path("taxonomy/rdp_species_assignment_16.fa.gz"), verbose=TRUE) # Assign species
saveRDS(taxa.rdp, file.path("out_filter1/taxa.rdp.rds"))
saveRDS(species.rdp, file.path("out_filter1/species.rdp.rds"))

message("Assign taxonomy with GreenGenes")
taxa.greeng <- assignTaxonomy(seqtab, file.path("taxonomy/gg_13_8_train_set_97.fa.gz"), minBoot = 80, multithread=TRUE, verbose=TRUE)
saveRDS(taxa.greeng, file.path("out_filter1/taxa.greeng.rds"))

message("Save files, tack, taxa, species")
dir.create(file.path("final.data_filter1"))

seq.abundances <- as.data.frame(t(seqtab))
seq.abundances <- setDT(as.data.frame(seq.abundances), keep.rownames = TRUE)
colnames(seq.abundances)[colnames(seq.abundances) == c("rn")] <- c("ASV_seq")
seq.abundances <- setDT(as.data.frame(seq.abundances), keep.rownames = TRUE)
seq.abundances$rn <- paste0("ASV_", seq.abundances$rn)
colnames(seq.abundances)[colnames(seq.abundances) == c("rn")] <- c("ASV")

message("Silva")
taxa1.silva <- setDT(as.data.frame(taxa.silva), keep.rownames = TRUE)
colnames(taxa1.silva)[colnames(taxa1.silva) == c("rn")] <- c("ASV_seq")
taxa.abundance.silva <- merge(seq.abundances, taxa1.silva, by = "ASV_seq")
write.table(taxa.abundance.silva, file = file.path("final.data_filter1/taxa.silva.txt"), quote = FALSE, sep = "\t", row.names = FALSE)

species1.silva <- setDT(as.data.frame(species.silva), keep.rownames = TRUE)
colnames(species1.silva)[colnames(species1.silva) == c("rn")] <- c("ASV_seq")
species.abundance.silva <- merge(seq.abundances, species1.silva, by = "ASV_seq")
write.table(species.abundance.silva, file = file.path("final.data_filter1/species.silva.txt"), quote = FALSE, sep = "\t", row.names = FALSE)

message("RDP")
taxa1.rdp <- setDT(as.data.frame(taxa.rdp), keep.rownames = TRUE)
colnames(taxa1.rdp)[colnames(taxa1.rdp) == c("rn")] <- c("ASV_seq")
taxa.abundance.rdp <- merge(seq.abundances, taxa1.rdp, by = "ASV_seq")
write.table(taxa.abundance.rdp, file = file.path("final.data_filter1/taxa.rdp.txt"), quote = FALSE, sep = "\t", row.names = FALSE)

species1.rdp <- setDT(as.data.frame(species.rdp), keep.rownames = TRUE)
colnames(species1.rdp)[colnames(species1.rdp) == c("rn")] <- c("ASV_seq")
species.abundance.rdp <- merge(seq.abundances, species1.rdp, by = "ASV_seq")
write.table(species.abundance.rdp, file = file.path("final.data_filter1/species.rdp.txt"), quote = FALSE, sep = "\t", row.names = FALSE)

message("GreenGeenes")
taxa1.greeng <- setDT(as.data.frame(taxa.greeng), keep.rownames = TRUE)
colnames(taxa1.greeng)[colnames(taxa1.greeng) == c("rn")] <- c("ASV_seq")
taxa.abundance.greeng <- merge(seq.abundances, taxa1.greeng, by = "ASV_seq")
write.table(taxa.abundance.greeng, file = file.path("final.data_filter1/taxa.greeng.txt"), quote = FALSE, sep = "\t", row.names = FALSE)

